==> slang_2025.6.1~rc1-1lunarg22.04.orig.tar.gz: slang-2025.6.1~rc1-1lunarg22.04/external/slang-rhi/tests/texture-utils.cpp <==
#include "texture-utils.h"

#include <slang-com-ptr.h>

#include <stdio.h>
#include <stdlib.h>

#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable : 4996)
#endif
#define STB_IMAGE_WRITE_IMPLEMENTATION
#include "stb_image_write.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif

namespace rhi::testing {

TextureInfo::~TextureInfo()
{
    for (SubresourceData subresourceData : subresourceDatas)
    {
        ::free((void*)subresourceData.data);
    }
}

Size getTexelSize(Format format)
{
    const FormatInfo& info = getFormatInfo(format);
    return info.blockSizeInBytes / info.pixelsPerBlock;
}

uint32_t getSubresourceIndex(uint32_t mipLevel, uint32_t mipLevelCount, uint32_t baseArrayLayer)
{
    return baseArrayLayer * mipLevelCount + mipLevel;
}

RefPtr<ValidationTextureFormatBase> getValidationTextureFormat(Format format)
{
    switch (format)
    {
    case Format::R32G32B32A32_TYPELESS:
        return new ValidationTextureFormat<uint32_t>(4);
    case Format::R32G32B32_TYPELESS:
        return new ValidationTextureFormat<uint32_t>(3);
    case Format::R32G32_TYPELESS:
        return new ValidationTextureFormat<uint32_t>(2);
    case Format::R32_TYPELESS:
        return new ValidationTextureFormat<uint32_t>(1);

    case Format::R16G16B16A16_TYPELESS:
        return new ValidationTextureFormat<uint16_t>(4);
    case Format::R16G16_TYPELESS:
        return new ValidationTextureFormat<uint16_t>(2);
    case Format::R16_TYPELESS:
        return new ValidationTextureFormat<uint16_t>(1);

    case Format::R8G8B8A8_TYPELESS:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::R8G8_TYPELESS:
        return new ValidationTextureFormat<uint8_t>(2);
    case Format::R8_TYPELESS:
        return new ValidationTextureFormat<uint8_t>(1);
    case Format::B8G8R8A8_TYPELESS:
        return new ValidationTextureFormat<uint8_t>(4);

    case Format::R32G32B32A32_FLOAT:
        return new ValidationTextureFormat<float>(4);
    case Format::R32G32B32_FLOAT:
        return new ValidationTextureFormat<float>(3);
    case Format::R32G32_FLOAT:
        return new ValidationTextureFormat<float>(2);
    case Format::R32_FLOAT:
        return new ValidationTextureFormat<float>(1);

    case Format::R16G16B16A16_FLOAT:
        return new ValidationTextureFormat<uint16_t>(4);
    case Format::R16G16_FLOAT:
        return new ValidationTextureFormat<uint16_t>(2);
    case Format::R16_FLOAT:
        return new ValidationTextureFormat<uint16_t>(1);

    case Format::R64_UINT:
        return new ValidationTextureFormat<uint64_t>(1);

    case Format::R32G32B32A32_UINT:
        return new ValidationTextureFormat<uint32_t>(4);
    case Format::R32G32B32_UINT:
        return new ValidationTextureFormat<uint32_t>(3);
    case Format::R32G32_UINT:
        return new ValidationTextureFormat<uint32_t>(2);
    case Format::R32_UINT:
        return new ValidationTextureFormat<uint32_t>(1);

    case Format::R16G16B16A16_UINT:
        return new ValidationTextureFormat<uint16_t>(4);
    case Format::R16G16_UINT:
        return new ValidationTextureFormat<uint16_t>(2);
    case Format::R16_UINT:
        return new ValidationTextureFormat<uint16_t>(1);

    case Format::R8G8B8A8_UINT:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::R8G8_UINT:
        return new ValidationTextureFormat<uint8_t>(2);
    case Format::R8_UINT:
        return new ValidationTextureFormat<uint8_t>(1);

    case Format::R64_SINT:
        return new ValidationTextureFormat<int64_t>(1);

    case Format::R32G32B32A32_SINT:
        return new ValidationTextureFormat<int32_t>(4);
    case Format::R32G32B32_SINT:
        return new ValidationTextureFormat<int32_t>(3);
    case Format::R32G32_SINT:
        return new ValidationTextureFormat<int32_t>(2);
    case Format::R32_SINT:
        return new ValidationTextureFormat<int32_t>(1);

    case Format::R16G16B16A16_SINT:
        return new ValidationTextureFormat<int16_t>(4);
    case Format::R16G16_SINT:
        return new ValidationTextureFormat<int16_t>(2);
    case Format::R16_SINT:
        return new ValidationTextureFormat<int16_t>(1);

    case Format::R8G8B8A8_SINT:
        return new ValidationTextureFormat<int8_t>(4);
    case Format::R8G8_SINT:
        return new ValidationTextureFormat<int8_t>(2);
    case Format::R8_SINT:
        return new ValidationTextureFormat<int8_t>(1);

    case Format::R16G16B16A16_UNORM:
        return new ValidationTextureFormat<uint16_t>(4);
    case Format::R16G16_UNORM:
        return new ValidationTextureFormat<uint16_t>(2);
    case Format::R16_UNORM:
        return new ValidationTextureFormat<uint16_t>(1);

    case Format::R8G8B8A8_UNORM:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::R8G8B8A8_UNORM_SRGB:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::R8G8_UNORM:
        return new ValidationTextureFormat<uint8_t>(2);
    case Format::R8_UNORM:
        return new ValidationTextureFormat<uint8_t>(1);
    case Format::B8G8R8A8_UNORM:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::B8G8R8A8_UNORM_SRGB:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::B8G8R8X8_UNORM:
        return new ValidationTextureFormat<uint8_t>(3);
    case Format::B8G8R8X8_UNORM_SRGB:
        return new ValidationTextureFormat<uint8_t>(3);

    case Format::R16G16B16A16_SNORM:
        return new ValidationTextureFormat<int16_t>(4);
    case Format::R16G16_SNORM:
        return new ValidationTextureFormat<int16_t>(2);
    case Format::R16_SNORM:
        return new ValidationTextureFormat<int16_t>(1);

    case Format::R8G8B8A8_SNORM:
        return new ValidationTextureFormat<int8_t>(4);
    case Format::R8G8_SNORM:
        return new ValidationTextureFormat<int8_t>(2);
    case Format::R8_SNORM:
        return new ValidationTextureFormat<int8_t>(1);

    case Format::D32_FLOAT:
        return new ValidationTextureFormat<float>(1);
    case Format::D16_UNORM:
        return new ValidationTextureFormat<uint16_t>(1);

    case Format::B4G4R4A4_UNORM:
        return new PackedValidationTextureFormat<uint16_t>(4, 4, 4, 4);
    case Format::B5G6R5_UNORM:
        return new PackedValidationTextureFormat<uint16_t>(5, 6, 5, 0);
    case Format::B5G5R5A1_UNORM:
        return new PackedValidationTextureFormat<uint16_t>(5, 5, 5, 1);

    case Format::R9G9B9E5_SHAREDEXP:
        return new ValidationTextureFormat<uint32_t>(1);
    case Format::R10G10B10A2_TYPELESS:
        return new PackedValidationTextureFormat<uint32_t>(10, 10, 10, 2);
    case Format::R10G10B10A2_UNORM:
        return new PackedValidationTextureFormat<uint32_t>(10, 10, 10, 2);
    case Format::R10G10B10A2_UINT:
        return new PackedValidationTextureFormat<uint32_t>(10, 10, 10, 2);
    case Format::R11G11B10_FLOAT:
        return new PackedValidationTextureFormat<uint32_t>(11, 11, 10, 0);

        // TODO: Add testing support for BC formats
        //                     BC1_UNORM,
        //                     BC1_UNORM_SRGB,
        //                     BC2_UNORM,
        //                     BC2_UNORM_SRGB,
        //                     BC3_UNORM,
        //                     BC3_UNORM_SRGB,
        //                     BC4_UNORM,
        //                     BC4_SNORM,
        //                     BC5_UNORM,
        //                     BC5_SNORM,
        //                     BC6H_UF16,
        //                     BC6H_SF16,
        //                     BC7_UNORM,
        //                     BC7_UNORM_SRGB,
    default:
        return nullptr;
    }
}

void generateTextureData(RefPtr<TextureInfo> texture, ValidationTextureFormatBase* validationFormat)
{
    auto extents = texture->extents;
    auto arrayLayers = texture->arrayLayerCount;
    auto mipLevels = texture->mipLevelCount;
    auto texelSize = getTexelSize(texture->format);

    for (uint32_t layer = 0; layer < arrayLayers; ++layer)
    {
        for (uint32_t mip = 0; mip < mipLevels; ++mip)
        {
            RefPtr<ValidationTextureData> subresource = new ValidationTextureData();

            auto mipWidth = std::max(extents.width >> mip, 1);
            auto mipHeight = std::max(extents.height >> mip, 1);
            auto mipDepth = std::max(extents.depth >> mip, 1);
            auto mipSize = mipWidth * mipHeight * mipDepth * texelSize;
            subresource->textureData = ::malloc(mipSize);
            REQUIRE(subresource->textureData != nullptr);

            subresource->extents.width = mipWidth;
            subresource->extents.height = mipHeight;
            subresource->extents.depth = mipDepth;
            subresource->strides.x = texelSize;
            subresource->strides.y = mipWidth * texelSize;
            subresource->strides.z = mipHeight * subresource->strides.y;
            texture->subresourceObjects.push_back(subresource);

            for (int z = 0; z < mipDepth; ++z)
            {
                for (int y = 0; y < mipHeight; ++y)
                {
                    for (int x = 0; x < mipWidth; ++x)
                    {
                        auto texel = subresource->getBlockAt(x, y, z);
                        validationFormat->initializeTexel(texel, x, y, z, mip, layer);
                    }
                }
            }

            SubresourceData subData = {};
            subData.data = subresource->textureData;
            subData.strideY = subresource->strides.y;
            subData.strideZ = subresource->strides.z;
            texture->subresourceDatas.push_back(subData);
        }
    }
}

std::vector<uint8_t> removePadding(ISlangBlob* pixels, uint32_t width, uint32_t height, Size rowPitch, Size pixelSize)
{
    std::vector<uint8_t> buffer;
    buffer.resize(height * rowPitch);
    for (uint32_t i = 0; i < height; ++i)
    {
        Offset srcOffset = i * rowPitch;
        Offset dstOffset = i * width * pixelSize;
        memcpy(buffer.data() + dstOffset, (char*)pixels->getBufferPointer() + srcOffset, width * pixelSize);
    }

    return buffer;
}

Result writeImage(const char* filename, ISlangBlob* pixels, uint32_t width, uint32_t height)
{
    int stbResult = stbi_write_hdr(filename, width, height, 4, (float*)pixels->getBufferPointer());

    return stbResult ? SLANG_OK : SLANG_FAIL;
}

Result writeImage(
    const char* filename,
    ISlangBlob* pixels,
    uint32_t width,
    uint32_t height,
    uint32_t rowPitch,
    uint32_t pixelSize
)
{
    if (rowPitch == width * pixelSize)
        return writeImage(filename, pixels, width, height);

    std::vector<uint8_t> buffer = removePadding(pixels, width, height, rowPitch, pixelSize);

    int stbResult = stbi_write_hdr(filename, width, height, 4, (float*)buffer.data());

    return stbResult ? SLANG_OK : SLANG_FAIL;
}

} // namespace rhi::testing

==> slang_2025.7.1~rc1-1lunarg22.04.orig.tar.gz: slang-2025.7.1~rc1-1lunarg22.04/external/slang-rhi/tests/texture-utils.cpp <==
#include "texture-utils.h"

#include <slang-com-ptr.h>

#include <stdio.h>
#include <stdlib.h>

#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable : 4996)
#endif
#define STB_IMAGE_WRITE_IMPLEMENTATION
#include "stb_image_write.h"
#ifdef _MSC_VER
#pragma warning(pop)
#endif

namespace rhi::testing {

TextureInfo::~TextureInfo()
{
    for (SubresourceData subresourceData : subresourceDatas)
    {
        ::free((void*)subresourceData.data);
    }
}

Size getTexelSize(Format format)
{
    const FormatInfo& info = getFormatInfo(format);
    return info.blockSizeInBytes / info.pixelsPerBlock;
}

RefPtr<ValidationTextureFormatBase> getValidationTextureFormat(Format format)
{
    switch (format)
    {
    case Format::RGBA32Float:
        return new ValidationTextureFormat<float>(4);
    case Format::RGB32Float:
        return new ValidationTextureFormat<float>(3);
    case Format::RG32Float:
        return new ValidationTextureFormat<float>(2);
    case Format::R32Float:
        return new ValidationTextureFormat<float>(1);

    case Format::RGBA16Float:
        return new ValidationTextureFormat<uint16_t>(4);
    case Format::RG16Float:
        return new ValidationTextureFormat<uint16_t>(2);
    case Format::R16Float:
        return new ValidationTextureFormat<uint16_t>(1);

    case Format::R64Uint:
        return new ValidationTextureFormat<uint64_t>(1);

    case Format::RGBA32Uint:
        return new ValidationTextureFormat<uint32_t>(4);
    case Format::RGB32Uint:
        return new ValidationTextureFormat<uint32_t>(3);
    case Format::RG32Uint:
        return new ValidationTextureFormat<uint32_t>(2);
    case Format::R32Uint:
        return new ValidationTextureFormat<uint32_t>(1);

    case Format::RGBA16Uint:
        return new ValidationTextureFormat<uint16_t>(4);
    case Format::RG16Uint:
        return new ValidationTextureFormat<uint16_t>(2);
    case Format::R16Uint:
        return new ValidationTextureFormat<uint16_t>(1);

    case Format::RGBA8Uint:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::RG8Uint:
        return new ValidationTextureFormat<uint8_t>(2);
    case Format::R8Uint:
        return new ValidationTextureFormat<uint8_t>(1);

    case Format::R64Sint:
        return new ValidationTextureFormat<int64_t>(1);

    case Format::RGBA32Sint:
        return new ValidationTextureFormat<int32_t>(4);
    case Format::RGB32Sint:
        return new ValidationTextureFormat<int32_t>(3);
    case Format::RG32Sint:
        return new ValidationTextureFormat<int32_t>(2);
    case Format::R32Sint:
        return new ValidationTextureFormat<int32_t>(1);

    case Format::RGBA16Sint:
        return new ValidationTextureFormat<int16_t>(4);
    case Format::RG16Sint:
        return new ValidationTextureFormat<int16_t>(2);
    case Format::R16Sint:
        return new ValidationTextureFormat<int16_t>(1);

    case Format::RGBA8Sint:
        return new ValidationTextureFormat<int8_t>(4);
    case Format::RG8Sint:
        return new ValidationTextureFormat<int8_t>(2);
    case Format::R8Sint:
        return new ValidationTextureFormat<int8_t>(1);

    case Format::RGBA16Unorm:
        return new ValidationTextureFormat<uint16_t>(4);
    case Format::RG16Unorm:
        return new ValidationTextureFormat<uint16_t>(2);
    case Format::R16Unorm:
        return new ValidationTextureFormat<uint16_t>(1);

    case Format::RGBA8Unorm:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::RGBA8UnormSrgb:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::RG8Unorm:
        return new ValidationTextureFormat<uint8_t>(2);
    case Format::R8Unorm:
        return new ValidationTextureFormat<uint8_t>(1);
    case Format::BGRA8Unorm:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::BGRA8UnormSrgb:
        return new ValidationTextureFormat<uint8_t>(4);
    case Format::BGRX8Unorm:
        return new ValidationTextureFormat<uint8_t>(3);
    case Format::BGRX8UnormSrgb:
        return new ValidationTextureFormat<uint8_t>(3);

    case Format::RGBA16Snorm:
        return new ValidationTextureFormat<int16_t>(4);
    case Format::RG16Snorm:
        return new ValidationTextureFormat<int16_t>(2);
    case Format::R16Snorm:
        return new ValidationTextureFormat<int16_t>(1);

    case Format::RGBA8Snorm:
        return new ValidationTextureFormat<int8_t>(4);
    case Format::RG8Snorm:
        return new ValidationTextureFormat<int8_t>(2);
    case Format::R8Snorm:
        return new ValidationTextureFormat<int8_t>(1);

    case Format::D32Float:
        return new ValidationTextureFormat<float>(1);
    case Format::D16Unorm:
        return new ValidationTextureFormat<uint16_t>(1);

    case Format::BGRA4Unorm:
        return new PackedValidationTextureFormat<uint16_t>(4, 4, 4, 4);
    case Format::B5G6R5Unorm:
        return new PackedValidationTextureFormat<uint16_t>(5, 6, 5, 0);
    case Format::BGR5A1Unorm:
        return new PackedValidationTextureFormat<uint16_t>(5, 5, 5, 1);

    case Format::RGB9E5Ufloat:
        return new ValidationTextureFormat<uint32_t>(1);
    case Format::RGB10A2Unorm:
        return new PackedValidationTextureFormat<uint32_t>(10, 10, 10, 2);
    case Format::RGB10A2Uint:
        return new PackedValidationTextureFormat<uint32_t>(10, 10, 10, 2);
    case Format::R11G11B10Float:
        return new PackedValidationTextureFormat<uint32_t>(11, 11, 10, 0);

        // TODO: Add testing support for BC formats
        //                     BC1Unorm,
        //                     BC1UnormSrgb,
        //                     BC2Unorm,
        //                     BC2UnormSrgb,
        //                     BC3Unorm,
        //                     BC3UnormSrgb,
        //                     BC4Unorm,
        //                     BC4Snorm,
        //                     BC5Unorm,
        //                     BC5Snorm,
        //                     BC6HUfloat,
        //                     BC6HSfloat,
        //                     BC7Unorm,
        //                     BC7UnormSrgb,
    default:
        return nullptr;
    }
}

void generateTextureData(RefPtr<TextureInfo> texture, ValidationTextureFormatBase* validationFormat)
{
    Extent3D extent = texture->extent;
    uint32_t layerCount = texture->arrayLength;
    if (texture->textureType == TextureType::TextureCube)
        layerCount *= 6;
    uint32_t mipLevels = texture->mipCount;
    Size texelSize = getTexelSize(texture->format);

    for (uint32_t layer = 0; layer < layerCount; ++layer)
    {
        for (uint32_t mip = 0; mip < mipLevels; ++mip)
        {
            RefPtr<ValidationTextureData> subresource = new ValidationTextureData();

            uint32_t mipWidth = std::max(extent.width >> mip, 1u);
            uint32_t mipHeight = std::max(extent.height >> mip, 1u);
            uint32_t mipDepth = std::max(extent.depth >> mip, 1u);
            uint32_t mipSize = mipWidth * mipHeight * mipDepth * texelSize;
            subresource->textureData = ::malloc(mipSize);
            REQUIRE(subresource->textureData != nullptr);

            subresource->extent.width = mipWidth;
            subresource->extent.height = mipHeight;
            subresource->extent.depth = mipDepth;
            subresource->pitches.x = texelSize;
            subresource->pitches.y = mipWidth * texelSize;
            subresource->pitches.z = mipHeight * subresource->pitches.y;
            texture->subresourceObjects.push_back(subresource);

            for (int z = 0; z < mipDepth; ++z)
            {
                for (int y = 0; y < mipHeight; ++y)
                {
                    for (int x = 0; x < mipWidth; ++x)
                    {
                        auto texel = subresource->getBlockAt(x, y, z);
                        validationFormat->initializeTexel(texel, x, y, z, mip, layer);
                    }
                }
            }

            SubresourceData subData = {};
            subData.data = subresource->textureData;
            subData.rowPitch = subresource->pitches.y;
            subData.slicePitch = subresource->pitches.z;
            texture->subresourceDatas.push_back(subData);
        }
    }
}

std::vector<uint8_t> removePadding(ISlangBlob* pixels, uint32_t width, uint32_t height, Size rowPitch, Size pixelSize)
{
    std::vector<uint8_t> buffer;
    buffer.resize(height * rowPitch);
    for (uint32_t i = 0; i < height; ++i)
    {
        Offset srcOffset = i * rowPitch;
        Offset dstOffset = i * width * pixelSize;
        memcpy(buffer.data() + dstOffset, (char*)pixels->getBufferPointer() + srcOffset, width * pixelSize);
    }

    return buffer;
}

Result writeImage(const char* filename, ISlangBlob* pixels, uint32_t width, uint32_t height)
{
    int stbResult = stbi_write_hdr(filename, width, height, 4, (float*)pixels->getBufferPointer());

    return stbResult ? SLANG_OK : SLANG_FAIL;
}

Result writeImage(
    const char* filename,
    ISlangBlob* pixels,
    uint32_t width,
    uint32_t height,
    uint32_t rowPitch,
    uint32_t pixelSize
)
{
    if (rowPitch == width * pixelSize)
        return writeImage(filename, pixels, width, height);

    std::vector<uint8_t> buffer = removePadding(pixels, width, height, rowPitch, pixelSize);

    int stbResult = stbi_write_hdr(filename, width, height, 4, (float*)buffer.data());

    return stbResult ? SLANG_OK : SLANG_FAIL;
}

} // namespace rhi::testing
